package net.apunch.blacksmith.util;

import net.apunch.blacksmith.BlacksmithPlugin;
import net.citizensnpcs.api.util.DataKey;
import net.citizensnpcs.api.util.YamlStorage;

import java.io.File;

public class Settings {

    private String busyWithPlayerMessage = Setting.BUSY_WITH_PLAYER_MESSAGE.asString();
    private String busyReforgingMessage = Setting.BUSY_WITH_REFORGE_MESSAGE.asString();
    private String costMessage = Setting.COST_MESSAGE.asString();
    private String invalidItemMessage = Setting.INVALID_ITEM_MESSAGE.asString();
    private String startReforgeMessage = Setting.START_REFORGE_MESSAGE.asString();
    private String successMessage = Setting.SUCCESS_MESSAGE.asString();
    private String failMessage = Setting.FAIL_MESSAGE.asString();
    private String insufficientFundsMessage = Setting.INSUFFICIENT_FUNDS_MESSAGE.asString();
    private String coolDownUnexpiredMessage = Setting.COOL_DOWN_UNEXPIRED_MESSAGE.asString();
    private String itemChangedMessage = Setting.ITEM_UNEXPECTEDLY_CHANGED_MESSAGE.asString();
    private int minReforgeDelay = Setting.MIN_REFORGE_DELAY.asInt();
    private int maxReforgeDelay = Setting.MAX_REFORGE_DELAY.asInt();
    private int reforgeCoolDown = Setting.REFORGE_COOL_DOWN.asInt();
    private int failChance = Setting.FAIL_CHANCE.asInt();
    private int extraEnchantmentChance = Setting.EXTRA_ENCHANTMENT_CHANCE.asInt();
    private int maxEnchantments = Setting.MAX_ENCHANTMENTS.asInt();
    private boolean dropItem = Setting.DROP_ITEM.asBoolean();
    private boolean disableCoolDown = Setting.DISABLE_COOL_DOWN.asBoolean();
    private boolean disableDelay = Setting.DISABLE_DELAY.asBoolean();
    private final YamlStorage config;

    public Settings(BlacksmithPlugin plugin) {
        config = new YamlStorage(new File(plugin.getDataFolder() + File.separator + "config.yml"), "Blacksmith Configuration");
    }

    public void load() {
        config.load();
        DataKey root = config.getKey("");
        for (Setting setting : Setting.values()) {
            if (!root.keyExists(setting.getPath())) {
                root.setRaw(setting.getPath(), setting.get());
            } else {
                setting.set(root.getRaw(setting.getPath()));
            }
        }

        config.save();
    }

    public YamlStorage getConfig() {
        return config;
    }

    public void loadVariables(DataKey key) {
        // Override defaults if they exist
        if (key.keyExists("messages.busy-with-player")) {
            busyWithPlayerMessage = key.getString("messages.busy-with-player");
        }
        if (key.keyExists("messages.busy-with-reforge")) {
            busyReforgingMessage = key.getString("messages.busy-with-reforge");
        }
        if (key.keyExists("messages.cost")) {
            costMessage = key.getString("messages.cost");
        }
        if (key.keyExists("messages.invalid-item")) {
            invalidItemMessage = key.getString("messages.invalid-item");
        }
        if (key.keyExists("messages.start-reforge")) {
            startReforgeMessage = key.getString("messages.start-reforge");
        }
        if (key.keyExists("messages.successful-reforge")) {
            successMessage = key.getString("messages.successful-reforge");
        }
        if (key.keyExists("messages.fail-reforge")) {
            failMessage = key.getString("messages.fail-reforge");
        }
        if (key.keyExists("messages.insufficient-funds")) {
            insufficientFundsMessage = key.getString("messages.insufficient-funds");
        }
        if (key.keyExists("messages.cooldown-not-expired")) {
            coolDownUnexpiredMessage = key.getString("messages.cooldown-not-expired");
        }
        if (key.keyExists("messages.item-changed-during-reforge")) {
            itemChangedMessage = key.getString("messages.item-changed-during-reforge");
        }
        if (key.keyExists("delays-in-seconds.minimum")) {
            minReforgeDelay = key.getInt("delays-in-seconds.minimum");
        }
        if (key.keyExists("delays-in-seconds.maximum")) {
            maxReforgeDelay = key.getInt("delays-in-seconds.maximum");
        }
        if (key.keyExists("delays-in-seconds.reforge-cooldown")) {
            reforgeCoolDown = key.getInt("delays-in-seconds.reforge-cooldown");
        }
        if (key.keyExists("percent-chance-to-fail-reforge")) {
            failChance = key.getInt("percent-chance-to-fail-reforge");
        }
        if (key.keyExists("maximum-enchantments")) {
            maxEnchantments = key.getInt("maximum-enchantments");
        }
        if (key.keyExists("extra-enchantments-chance")) {
            extraEnchantmentChance = key.getInt("extra-enchantment-chance");
        }
        if (key.keyExists("dropitem")) {
            dropItem = key.getBoolean("dropitem");
        }
        if (key.keyExists("disable-cooldown")) {
            disableCoolDown = key.getBoolean("disable-cooldown");
        }
        if (key.keyExists("disable-delay")) {
            disableDelay = key.getBoolean("disable-delay");
        }
    }

    public void saveVariables(DataKey key) {
        key.setString("messages.busy-with-player", getBusyWithPlayerMessage());
        key.setString("messages.busy-with-reforge", getBusyReforgingMessage());
        key.setString("messages.cost", getCostMessage());
        key.setString("messages.invalid-item", getInvalidItemMessage());
        key.setString("messages.start-reforge", getStartReforgeMessage());
        key.setString("messages.successful-reforge", getSuccessMessage());
        key.setString("messages.fail-reforge", getFailMessage());
        key.setString("messages.insufficient-funds", getInsufficientFundsMessage());
        key.setString("messages.cooldown-not-expired", getCoolDownUnexpiredMessage());
        key.setString("messages.item-changed-during-reforge", getItemChangedMessage());
        key.setInt("delays-in-seconds.minimum", getMinReforgeDelay());
        key.setInt("delays-in-seconds.maximum", getMaxReforgeDelay());
        key.setInt("delays-in-seconds.reforge-cooldown", getReforgeCoolDown());
        key.setInt("percent-chance-to-fail-reforge", getFailChance());
        key.setInt("percent-chance-for-extra-enchantment", getExtraEnchantmentChance());
        key.setInt("maximum-enchantments", getMaxEnchantments());
        key.setBoolean("drop-item", getDropItem());
        key.setBoolean("disable-delay", getDisableDelay());
        key.setBoolean("disable-cooldown", getDisableCoolDown());
    }

    public String getBusyWithPlayerMessage() {
        return busyWithPlayerMessage;
    }

    public String getBusyReforgingMessage() {
        return busyReforgingMessage;
    }

    public String getCostMessage() {
        return costMessage;
    }

    public String getInvalidItemMessage() {
        return invalidItemMessage;
    }

    public String getStartReforgeMessage() {
        return startReforgeMessage;
    }

    public String getSuccessMessage() {
        return successMessage;
    }

    public String getFailMessage() {
        return failMessage;
    }

    public String getInsufficientFundsMessage() {
        return insufficientFundsMessage;
    }

    public String getCoolDownUnexpiredMessage() {
        return coolDownUnexpiredMessage;
    }

    public String getItemChangedMessage() {
        return itemChangedMessage;
    }

    public int getMinReforgeDelay() {
        return minReforgeDelay;
    }

    public int getMaxReforgeDelay() {
        return maxReforgeDelay;
    }

    public int getReforgeCoolDown() {
        return reforgeCoolDown;
    }

    public int getFailChance() {
        return failChance;
    }

    public int getExtraEnchantmentChance() {
        return extraEnchantmentChance;
    }

    public int getMaxEnchantments() {
        return maxEnchantments;
    }

    public boolean getDropItem() {
        return dropItem;
    }

    public boolean getDisableCoolDown() {
        return disableCoolDown;
    }

    public boolean getDisableDelay() {
        return disableDelay;
    }

}